﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms;

using Infragistics.Win;
using Infragistics.Win.Misc;
using Infragistics.Win.UltraWinGrid;

using ADEC.Common.Graphics;
using ADEC.Common.Module;
using ADEC.Common.Utilities;

namespace GridControl
{
    /// <summary>
    /// Represents a user control for managing the message headers for the Inbox,
    /// Sent Items, and Deleted Items.
    /// </summary>
    public partial class ucMessageHeaders : UserControl
    {
        private const string FROM_COLUMN_KEY = "SENDER_NAME";
        private const string SUBJECT_COLUMN_KEY = "MESSAGE_SUBJECT";
        private const string SIZE_COLUMN_KEY = "Size";
        private const string RECEIVED_COLUMN_KEY = "Received";
        private const string IMPORTANCE_LEVEL_COLUMN_KEY = "Importance";
        private const string ATTACHMENT_COLUMN_KEY = "Attachment";
        private const string MAIL_COLUMN_KEY = "MailIcon";

        private SENDS_MESSAGE_HEADER_INBOX_V_COLLECTION _list = new SENDS_MESSAGE_HEADER_INBOX_V_COLLECTION();

        /// <summary>
        /// Provides the ability to initialize the instance of the class.
        /// </summary>
        public ucMessageHeaders()
        {
            this.SetStyle(ControlStyles.AllPaintingInWmPaint, true);
            this.SetStyle(ControlStyles.OptimizedDoubleBuffer, true);

            InitializeComponent();
            UiHelper.InitializeGridSettings(ultraGridMessages);
            PopulateCollection();
        }

        public void SetDataSource()
        {
            ultraGridMessages.DataSource = null;
            ultraGridMessages.Layouts.Clear();
            ultraGridMessages.DataSource = _list;
        }

        private void ultraGridMessages_InitializeLayout(object sender, Infragistics.Win.UltraWinGrid.InitializeLayoutEventArgs e)
        {
            e.Layout.Bands[0].Columns.Add(IMPORTANCE_LEVEL_COLUMN_KEY);
            e.Layout.Bands[0].Columns.Add(MAIL_COLUMN_KEY);
            e.Layout.Bands[0].Columns.Add(ATTACHMENT_COLUMN_KEY);
            e.Layout.Bands[0].Columns.Add(RECEIVED_COLUMN_KEY);
            e.Layout.Bands[0].Columns.Add(SIZE_COLUMN_KEY);

            //ultraGridMessages.DisplayLayout.Bands[0].Columns[ID_COLUMN_KEY].Hidden = true;
            SetColumnPosition(e.Layout.Bands[0].Columns);
            SetColumnAttributes(e.Layout.Bands[0].Columns);
            HideColumns(e.Layout.Bands[0].Columns);
        }

        /// <summary>
        /// Sets the visible position of the columns that are shown.
        /// </summary>
        /// <param name="columns">Collection of column objects.</param>
        private void SetColumnPosition(ColumnsCollection columns)
        {
            columns[IMPORTANCE_LEVEL_COLUMN_KEY].Header.VisiblePosition = 0;
            columns[MAIL_COLUMN_KEY].Header.VisiblePosition = 1;
            columns[ATTACHMENT_COLUMN_KEY].Header.VisiblePosition = 2;
            columns[FROM_COLUMN_KEY].Header.VisiblePosition = 3;
            columns[SUBJECT_COLUMN_KEY].Header.VisiblePosition = 4;
            columns[RECEIVED_COLUMN_KEY].Header.VisiblePosition = 5;
            columns[SIZE_COLUMN_KEY].Header.VisiblePosition = 6;
        }

        /// <summary>
        /// Sets the column attributes of the columns that are shown.
        /// </summary>
        /// <param name="columns">Collection of column objects.</param>
        private void SetColumnAttributes(ColumnsCollection columns)
        {
            UiHelper.SetColumnAttributes(columns[IMPORTANCE_LEVEL_COLUMN_KEY], 20, Images.GetImage(Images.IEImages.OP._24x24.Shadow.flag_yellow));
            UiHelper.SetColumnAttributes(columns[MAIL_COLUMN_KEY], 20, Images.GetImage(Images.IEImages.AB._24x24.Shadow.document_plain));
            UiHelper.SetColumnAttributes(columns[ATTACHMENT_COLUMN_KEY], 20, Images.GetImage(Images.IEImages.BD._24x24.Shadow.paperclip));

            UiHelper.SetColumnAttributes(columns[SIZE_COLUMN_KEY], 35, HAlign.Right, "Size");
            UiHelper.SetColumnAttributes(columns[SUBJECT_COLUMN_KEY], 200, HAlign.Default, "Subject");
            UiHelper.SetColumnAttributes(columns[FROM_COLUMN_KEY], 120, HAlign.Default, "From");
            UiHelper.SetColumnAttributes(columns[RECEIVED_COLUMN_KEY], 65, HAlign.Right, "Received");
        }

        private void HideColumns(ColumnsCollection columns)
        {
            columns["MESSAGE_ID"].Hidden = true;
            columns["MESSAGE_IMPORTANCE_LEVEL"].Hidden = true;
            columns["SENDER_ID"].Hidden = true;
            columns["RESPOND_OPTION"].Hidden = true;
            columns["MARKED_UNREAD"].Hidden = true;
            columns["DATE_RECEIVED"].Hidden = true;
            columns["ATTACHMENTS_SIZE"].Hidden = true;
            columns["AlertDisplayed"].Hidden = true;
        }

        /// <summary>
        /// Event that occurs when the row is double-clicked. 
        /// </summary>
        /// <param name="sender">Reference to the object that called the event handler.</param>
        /// <param name="e">Class instance containing data pertinent to the event.</param>
        private void ultraGridMessages_DoubleClickRow(object sender, DoubleClickRowEventArgs e)
        {

        }

        private void ultraGridMessages_InitializeRow(object sender, InitializeRowEventArgs e)
        {
            e.Row.Cells[IMPORTANCE_LEVEL_COLUMN_KEY].Value =
                GetMessageImportanceLevelImage((int)e.Row.Cells["MESSAGE_IMPORTANCE_LEVEL"].Value);

            e.Row.Cells[MAIL_COLUMN_KEY].Value = GetMailImage((int)e.Row.Cells["RESPOND_OPTION"].Value, (bool)e.Row.Cells["MARKED_UNREAD"].Value);

            e.Row.Cells[ATTACHMENT_COLUMN_KEY].Value = GetAttachmentsImage((long)e.Row.Cells["ATTACHMENTS_SIZE"].Value > 0);

            if (e.Row.Cells["DATE_RECEIVED"].Value != null)
            {
                e.Row.Cells[RECEIVED_COLUMN_KEY].Value = ((DateTime)e.Row.Cells["DATE_RECEIVED"].Value).ToADEC_LongDateTime();
            }

            e.Row.Cells[SIZE_COLUMN_KEY].Value = FormatHelper.FormatBytes((long)e.Row.Cells["ATTACHMENTS_SIZE"].Value);
            e.Row.CellAppearance.FontData.Bold = (bool)e.Row.Cells["MARKED_UNREAD"].Value ? DefaultableBoolean.True : DefaultableBoolean.False;
        }

        private Bitmap GetMailImage(int respondOption, bool isUnread)
        {
            Bitmap image = null;

            switch (respondOption)
            {
                case RespondOption.NONE:
                    if (isUnread)
                    {
                        // TODO: Need to replace with a different image
                        image = Images.GetImage(Images.IEImages.NS._16x16.Shadow.mail2);
                    }
                    else
                    {
                        image = Images.GetImage(Images.IEImages.NS._16x16.Shadow.mail);
                    }
                    break;

                case RespondOption.REPLY:
                    break;

                case RespondOption.REPLY_ALL:
                    break;

                case RespondOption.FORWARD:
                    break;

                default:
                    // Let it default to None, but log it as a warning
                    image = Images.GetImage(Images.IEImages.NS._16x16.Shadow.mail);
                    break;
            }

            return image;
        }

        /// <summary>
        /// Gets the image for the message importance level column.
        /// </summary>
        /// <param name="importanceLevel">The importance level of the message.</param>
        /// <returns>The image to display or null.</returns>
        private Bitmap GetMessageImportanceLevelImage(int importanceLevel)
        {
            Bitmap image = null;

            switch (importanceLevel)
            {
                case MessageImportanceLevel.LOW:
                    image = Images.GetImage(Images.IEImages.AB._16x16.Shadow.arrow_down_blue);
                    break;

                case MessageImportanceLevel.NORMAL:
                    break;

                case MessageImportanceLevel.HIGH:
                    image = Images.GetImage(Images.IEImages.OP._16x16.Shadow.flag_red);
                    break;

                default:
                    break;
            }

            return image;
        }

        /// <summary>
        /// Gets the image of the row if the message has attachments.
        /// </summary>
        /// <param name="hasAttachments">Flag indicating whether the message has attachments.</param>
        /// <returns>The image to display or null.</returns>
        private Bitmap GetAttachmentsImage(bool hasAttachments)
        {
            Bitmap image = null;

            if (hasAttachments)
            {
                image = Images.GetImage(Images.IEImages.BD._16x16.Shadow.paperclip);
            }

            return image;
        }

        private void PopulateCollection()
        {
            var msg = new SENDS_MESSAGE_HEADER_INBOX_V();

            msg.MESSAGE_ID = Guid.NewGuid();
            msg.MESSAGE_IMPORTANCE_LEVEL = 1;
            msg.ATTACHMENTS_SIZE = 0;
            msg.DATE_RECEIVED = DateTime.UtcNow;
            msg.MARKED_UNREAD = false;
            msg.MESSAGE_SUBJECT = "message #1";
            msg.RESPOND_OPTION = 0;
            msg.SENDER_NAME = "Goofy 1";
            _list.Add(msg);

            msg = new SENDS_MESSAGE_HEADER_INBOX_V();
            msg.MESSAGE_ID = Guid.NewGuid();
            msg.MESSAGE_IMPORTANCE_LEVEL = 2;
            msg.ATTACHMENTS_SIZE = 4096;
            msg.DATE_RECEIVED = DateTime.UtcNow;
            msg.MARKED_UNREAD = true;
            msg.MESSAGE_SUBJECT = "message #2";
            msg.RESPOND_OPTION = 0;
            msg.SENDER_NAME = "Goofy 2";
            _list.Add(msg);

            msg = new SENDS_MESSAGE_HEADER_INBOX_V();
            msg.MESSAGE_ID = Guid.NewGuid();
            msg.MESSAGE_IMPORTANCE_LEVEL = 3;
            msg.ATTACHMENTS_SIZE = 0;
            msg.DATE_RECEIVED = DateTime.UtcNow;
            msg.MARKED_UNREAD = true;
            msg.MESSAGE_SUBJECT = "message #3";
            msg.RESPOND_OPTION = 0;
            msg.SENDER_NAME = "Goofy 3";
            _list.Add(msg);

            msg = new SENDS_MESSAGE_HEADER_INBOX_V();
            msg.MESSAGE_ID = Guid.NewGuid();
            msg.MESSAGE_IMPORTANCE_LEVEL = 1;
            msg.ATTACHMENTS_SIZE = 4096;
            msg.DATE_RECEIVED = DateTime.UtcNow;
            msg.MARKED_UNREAD = false;
            msg.MESSAGE_SUBJECT = "message #4";
            msg.RESPOND_OPTION = 0;
            msg.SENDER_NAME = "Goofy 4";
            _list.Add(msg);

            msg = new SENDS_MESSAGE_HEADER_INBOX_V();
            msg.MESSAGE_ID = Guid.NewGuid();
            msg.MESSAGE_IMPORTANCE_LEVEL = 2;
            msg.ATTACHMENTS_SIZE = 0;
            msg.DATE_RECEIVED = DateTime.UtcNow;
            msg.MARKED_UNREAD = false;
            msg.MESSAGE_SUBJECT = "message #5";
            msg.RESPOND_OPTION = 0;
            msg.SENDER_NAME = "Goofy 5";
            _list.Add(msg);
        }

        private void ultraButton1_Click(object sender, EventArgs e)
        {
            var msg = new SENDS_MESSAGE_HEADER_INBOX_V();

            msg.MESSAGE_ID = Guid.NewGuid();
            msg.MESSAGE_IMPORTANCE_LEVEL = 1;
            msg.ATTACHMENTS_SIZE = 0;
            msg.DATE_RECEIVED = DateTime.UtcNow;
            msg.MARKED_UNREAD = false;
            msg.MESSAGE_SUBJECT = "new message";
            msg.RESPOND_OPTION = 0;
            msg.SENDER_NAME = "Donald Duck";

            _list.Insert(0, msg);
            ultraGridMessages.Rows.Refresh(RefreshRow.ReloadData);
            //ultraGridMessages.DataBind();
        }
    }
}
